Encoding: UTF-8
Package: semTools
Version: 0.5-9
Title: Useful Tools for Structural Equation Modeling
Description: Provides miscellaneous tools for structural equation modeling, 
             many of which extend the 'lavaan' package. For example, latent
             interactions can be estimated using product indicators (Lin et al.,
             2010, <doi:10.1080/10705511.2010.488999>) and simple effects probed;
             analytical power analyses can be conducted (Jak et al., 2021,
             <doi:10.3758/s13428-020-01479-0>); and scale reliability
             can be estimated based on estimated factor-model parameters.
Authors@R: c(person(given = c("Terrence","D."), family = "Jorgensen", role = c("aut","cre"), email="TJorgensen314@gmail.com", comment = c(ORCID = "0000-0001-5111-6773")),
		person(given = "Sunthud", family = "Pornprasertmanit", role = "aut", email = "psunthud@gmail.com", comment = c(ORCID = "0009-0006-6887-9930")),
		person(given = c("Alexander","M."), family = "Schoemann", role = "aut", email="schoemanna@ecu.edu", comment = c(ORCID = "0000-0002-8479-8798")),
		person(given = "Yves", family = "Rosseel", role = "aut", email="Yves.Rosseel@UGent.be", comment = c(ORCID = "0000-0002-4129-4477")),
		person(given = "Patrick", family = "Miller", role = "ctb", email="pmille13@nd.edu"),
		person(given = "Corbin", family = "Quick", role = "ctb", email="corbinq@umich.edu"),
		person(given = "Mauricio", family = "Garnier-Villarreal", role = "ctb", email="m.garniervillarreal@vu.nl", comment = c(ORCID = "0000-0002-2951-6647")),
		person(given = "James", family = "Selig", role = "ctb", email="selig@unm.edu"),
		person(given = "Aaron", family = "Boulton", role = "ctb", email="aboulton@email.unc.edu"),
		person(given = "Kristopher", family = "Preacher", role = "ctb", email="kris.preacher@vanderbilt.edu"),
		person(given = "Donna", family = "Coffman", role = "ctb", email="dlc30@psu.edu"),
		person(given = "Mijke", family = "Rhemtulla", role = "ctb", email="mrhemtulla@ucdavis.edu", comment = c(ORCID = "0000-0003-2572-2424")),
		person(given = "Alexander", family = "Robitzsch", role = "ctb", email="a.robitzsch@bifie.at", comment = c(ORCID = "0000-0002-8226-3132")),
		person(given = "Craig", family = "Enders", role = "ctb", email="cenders@psych.ucla.edu"),
		person(given = "Ruben", family = "Arslan", role = "ctb", email="rubenarslan@gmail.com", comment = c(ORCID = "0000-0002-6670-5658")),
		person(given = "Bell", family = "Clinton", role = "ctb", email="clintonbell@ku.edu"),
		person(given = "Pavel", family = "Panko", role = "ctb", email="pavel.panko@ttu.edu"),
		person(given = "Edgar", family = "Merkle", role = "ctb", email="merklee@missouri.edu", comment = c(ORCID = "0000-0001-7158-0653")),
		person(given = "Steven", family = "Chesnut", role = "ctb", email="Steven.Chesnut@usm.edu"),
		person(given = "Jarrett", family = "Byrnes", role = "ctb", email="Jarrett.Byrnes@umb.edu"),
		person(given = c("Jason","D."), family = "Rights", role = "ctb", email="jrights@psych.ubc.ca"),
		person(given = "Ylenio", family = "Longo", role = "ctb", email="yleniolongo@gmail.com"),
		person(given = "Maxwell", family = "Mansolf", role = "ctb", email="mamansolf@gmail.com", comment = c(ORCID = "0000-0001-6861-8657")),
		person(given = "Mattan S.", family = "Ben-Shachar", role = "ctb", email="matanshm@post.bgu.ac.il", comment = c(ORCID = "0000-0002-4287-4801")),
		person(given = "Mikko", family = "Rönkkö", role = "ctb", email = "mikko.ronkko@jyu.fi", comment = c(ORCID = "0000-0001-7988-7609")),
		person(given = "Andrew R.", family = "Johnson", role = "ctb", email = "andrew.johnson@curtin.edu.au", comment = c(ORCID = "0000-0001-7000-8065")),
		person(given = "Leonard", family = "Vanbrabant", role = "ctb", email = "L.Vanbrabant@ggdwestbrabant.nl")
		)
Depends: R(>= 4.0), lavaan(>= 0.6-21), methods
Imports: graphics, pbivnorm, stats, utils
Suggests: blavaan, emmeans, lavaan.mi, MASS, mice, mnormt, parallel,
        restriktor, testthat
License: GPL (>= 2)
LazyData: yes
LazyLoad: yes
URL: https://github.com/simsem/semTools/wiki
BugReports: https://github.com/simsem/semTools/issues
Date: 2026-07-13
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-13 21:17:25 UTC; terrence
Author: Terrence D. Jorgensen [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5111-6773>),
  Sunthud Pornprasertmanit [aut] (ORCID:
    <https://orcid.org/0009-0006-6887-9930>),
  Alexander M. Schoemann [aut] (ORCID:
    <https://orcid.org/0000-0002-8479-8798>),
  Yves Rosseel [aut] (ORCID: <https://orcid.org/0000-0002-4129-4477>),
  Patrick Miller [ctb],
  Corbin Quick [ctb],
  Mauricio Garnier-Villarreal [ctb] (ORCID:
    <https://orcid.org/0000-0002-2951-6647>),
  James Selig [ctb],
  Aaron Boulton [ctb],
  Kristopher Preacher [ctb],
  Donna Coffman [ctb],
  Mijke Rhemtulla [ctb] (ORCID: <https://orcid.org/0000-0003-2572-2424>),
  Alexander Robitzsch [ctb] (ORCID:
    <https://orcid.org/0000-0002-8226-3132>),
  Craig Enders [ctb],
  Ruben Arslan [ctb] (ORCID: <https://orcid.org/0000-0002-6670-5658>),
  Bell Clinton [ctb],
  Pavel Panko [ctb],
  Edgar Merkle [ctb] (ORCID: <https://orcid.org/0000-0001-7158-0653>),
  Steven Chesnut [ctb],
  Jarrett Byrnes [ctb],
  Jason D. Rights [ctb],
  Ylenio Longo [ctb],
  Maxwell Mansolf [ctb] (ORCID: <https://orcid.org/0000-0001-6861-8657>),
  Mattan S. Ben-Shachar [ctb] (ORCID:
    <https://orcid.org/0000-0002-4287-4801>),
  Mikko Rönkkö [ctb] (ORCID: <https://orcid.org/0000-0001-7988-7609>),
  Andrew R. Johnson [ctb] (ORCID:
    <https://orcid.org/0000-0001-7000-8065>),
  Leonard Vanbrabant [ctb]
Maintainer: Terrence D. Jorgensen <TJorgensen314@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-13 23:40:02 UTC
Built: R 4.6.1; ; 2026-07-14 04:21:45 UTC; unix
