### Terrence D. Jorgensen
### Last updated 25 August 2018
### automatically create documentation for "deprecated" help page

#' @title Deprecated functions in package \pkg{semTools}.
#' @description The functions listed below are deprecated and will be defunct
#'   in the near future. When possible, alternative functions with similar
#'   functionality are also mentioned. Help pages for deprecated functions are
#'   available at `help("semTools-deprecated")`.
#' @name semTools-deprecated
#' @keywords internal
NULL

